package de.tu_darmstadt.tk.SmartHomeNetworkSim.core;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * Model of the smart home, which contains all the important parts of the simulation, like {@link SmartDevice} and their {@link Link}.
 *
 * @author Andreas T. Meyer-Berg
 */
public class Model {

	private List<SmartDevice> devices;
	private List<Link> connectionNetworks;
	private List<Connection> connections;
	
	/**
	 * Width of the smart home model, 0 <= device.x < width
	 */
	private int width;
	
	/**
	 * Height of the smart home model, 0 <= device.y < height
	 */
	private int height;
	
	/**
	 * Depth of the smart home model, 0 <= device.y < depth
	 */
	private int depth;
	
	/**
	 * Radius (including the middlePoinnt) of a smartDevice
	 * 
	 */
	private int device_visualization_radius = 17;
	
	/**
	 * Initializes the Model, with 3 default devices for testing purposes
	 */
	public Model() {
		
		setWidth(640);
		setHeight(480);
		setDepth(480);
		
		devices = new ArrayList<SmartDevice>();
		connectionNetworks = new ArrayList<Link>();
		connections = new LinkedList<Connection>();
	}

	/**
	 * @return the connectionNetworks
	 */
	public List<Link> getConnectionNetworks() {
		return connectionNetworks;
	}

	/**
	 * Adds network connection
	 * @param connectionNetwork the connectionNetwork to add
	 */
	public void addConnectionNetwork(Link connectionNetwork) {
		this.connectionNetworks.add(connectionNetwork);
	}

	/**
	 * @return the devices
	 */
	public List<SmartDevice> getDevices() {
		return devices;
	}

	/**
	 * @param device the smartDevice to add to the smart home
	 */
	public void addDevices(SmartDevice device) {
		this.devices.add(device);
	}

	/**
	 * @return the width
	 */
	public int getWidth() {
		return width;
	}

	/**
	 * @param width the width to set
	 */
	public void setWidth(int width) {
		this.width = width;
	}

	/**
	 * @return the height
	 */
	public int getHeight() {
		return height;
	}

	/**
	 * @param height the height to set
	 */
	public void setHeight(int height) {
		this.height = height;
	}

	/**
	 * @return the depth
	 */
	public int getDepth() {
		return depth;
	}

	/**
	 * @param depth the depth to set
	 */
	public void setDepth(int depth) {
		this.depth = depth;
	}

	/**
	 * @return the device_visualization_radius
	 */
	public int getDevice_visualization_radius() {
		return device_visualization_radius;
	}

	/**
	 * @param device_visualization_radius the device_visualization_radius to set
	 */
	public void setDevice_visualization_radius(int device_visualization_radius) {
		this.device_visualization_radius = device_visualization_radius;
	}
	
	/**
	 * @return the connections
	 */
	public List<Connection> getConnections() {
		return connections;
	}

	/**
	 * @param connection the connection to add
	 */
	public void addConnection(Connection connection) {
		this.connections.add(connection);
	}	
}
