package de.tu_darmstadt.tk.SmartHomeNetworkSim.core;

/**
 * Representation of connection EndPoints, which allows configuration of timings
 * and if it reacts to incoming traffic or even triggers new connections.
 * 
 * @author Andreas T. Meyer-Berg
 */
public class Port {

	/**
	 * A closed Port which does not react to incoming traffic
	 */
	public static final short CLOSED = 0;

	/**
	 * Open Port which reacts to incoming traffic
	 */
	public static final short OPEN = 1;

	/**
	 * Open Port which reacts to incoming traffic and sends packets every
	 * triggerInterval
	 */
	public static final short SENDING = 2;

	/**
	 * Status of the Port, wethere it is CLOSED, OPEN or SENDING
	 */
	private short status;

	/**
	 * SmartDevice which owns the port
	 */
	private SmartDevice owner;

	/**
	 * Connection this Port listens to
	 */
	private Connection connection;
	
	/**
	 * Constant time interval, which triggers new traffic every
	 * {@code triggerInterval} milliseconds.
	 */
	private long triggerInterval;

	/**
	 * Last timestep traffic was triggered
	 */
	private long lastTrigger;

	/**
	 * Time in milliseconds needed for responds to incoming 
	 */
	private short responseTime;

	/**
	 * Port number of this Port.
	 */
	private short portNumber;

	/**
	 * Creates a new Port for the given Device with the specified PortNumber
	 * @param device SmartDevice this port listens on
	 * @param portNumber Number of the Port
	 */
	public Port(SmartDevice device, short portNumber){
		status = CLOSED;
		owner = device;
		connection = null;
		triggerInterval = 1000;
		lastTrigger = 0;
		responseTime = 5;
		this.portNumber = portNumber;
	}
	
	
	/**
	 * @return the status
	 */
	public short getStatus() {
		return status;
	}

	/**
	 * @param status
	 *            the status to set
	 */
	public void setStatus(short status) {
		this.status = status;
	}

	/**
	 * @return the owner
	 */
	public SmartDevice getOwner() {
		return owner;
	}

	/**
	 * @param owner
	 *            the owner to set
	 */
	public void setOwner(SmartDevice owner) {
		this.owner = owner;
	}

	/**
	 * @return the triggerInterval
	 */
	public long getTriggerInterval() {
		return triggerInterval;
	}

	/**
	 * @param triggerInterval
	 *            the triggerInterval to set
	 */
	public void setTriggerInterval(long triggerInterval) {
		if(triggerInterval<=0)
			this.triggerInterval = (long)0;
		else
		    this.triggerInterval = triggerInterval;
	}

	/**
	 * @return the responseTime
	 */
	public short getResponseTime() {
		return responseTime;
	}

	/**
	 * @param responseTime
	 *            the responseTime to set
	 */
	public void setResponseTime(short responseTime) {
		if(responseTime<=0)
			this.responseTime = 0;
		else
			this.responseTime = responseTime;
	}

	/**
	 * @return the portNumber
	 */
	public short getPortNumber() {
		return portNumber;
	}

	/**
	 * @param portNumber
	 *            the portNumber to set
	 */
	public void setPortNumber(short portNumber) {
		this.portNumber = portNumber;
	}

	/**
	 * @return the last timestep the port sent a package
	 */
	public long getLastTrigger() {
		return lastTrigger;
	}

	/**
	 * @param lastTrigger the last timestep the port sent
	 */
	public void setLastTrigger(long lastTrigger) {
		this.lastTrigger = lastTrigger;
	}

	/**
	 * @return the connection
	 */
	public Connection getConnection() {
		return connection;
	}

	/**
	 * @param connection the connection to set
	 */
	public void setConnection(Connection connection) {
		this.connection = connection;
	}

}
