package de.tu_darmstadt.tk.SmartHomeNetworkSim.core;

/**
 * Manages the Simulation by calling the relevant methods.
 *
 * @author Andreas T. Meyer-Berg
 */
public class SimulationManager {
	Model model;
	long stepDuration = 100;
	/**
	 * Creates a new Simulationmanager
	 * 
	 * @param model Model that should be simulated
	 */
	public SimulationManager(Model model) {
		this.model = model;
	}

	/**
	 * Simulates one time step at a given time fur a given duration
	 * 
	 * @param startTime
	 *            Time the simulation interval starts in
	 *            System.currentTimeMillis() time
	 * @param duration
	 *            Duration of the simulation interval in milliseconds
	 */
	public void simulateTimeIntervall(long startTime, long duration){
		//Simulate all Links, and their connections
		model.getConnectionNetworks().forEach(d -> d.simulateTimeInterval(startTime, duration));
		//Simulate SmartDevices - if they need some logic
		model.getDevices().forEach(d -> d.simulateTimeStep(startTime, duration));
		//Store Packages/Export Packages etc. (for debug purposes)
		model.getConnectionNetworks().forEach(d->d.getPackets()
				.forEach(p-> 
				{
					if(p == null)
						System.out.println("Packet: Null");
					else 
						System.out.println(p.getTextualRepresentation());
				}));
		
		
	}
}
