package de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation;

import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Packet;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Port;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Protocol;

/**
 * Simple Implementation of a protocol, which sends 
 *
 * @author Andreas T. Meyer-Berg
 */
public class SimpleProtocol implements Protocol {

	/**
	 * Source of the protocol, and destination which responds
	 */
	Port source, destination;
	
	/**
	 * True if the next Package is send by source, false if the next one will be send by destination
	 */
	boolean srcSends;
	
	public SimpleProtocol(Port src, Port dest){
		source = src;
		destination = dest;
		srcSends = true;
	}
	@Override
	public Collection<Packet> generateNextPakets(Port p, long timestep, boolean packetLost) {
		LinkedList<Packet> ret = new LinkedList<Packet>();
		if(p==null)return ret;
		p.setLastTrigger(timestep);
		if(packetLost)ret.add(new SimplePacket(timestep, p, p==source?destination:source, "Lost"));
		if(p == destination)
			ret.add(new SimplePacket(timestep, destination, source));
		else
			ret.add(new SimplePacket(timestep, source, destination));
		return ret;
	}
	
	@Override
	public int getNumberOfRoles() {
		// two different roles
		return 2;
	}
	
	@Override
	public String[] getRoles() {
		return new String[]{"Sender", "Receiver"};
	}
	
	@Override
	public Collection<Port> getDevicesWithRole(int role) {
		Port ret = null;
		switch (role) {
		case 0:
			ret = source;
			break;
		case 1:
			ret = destination;
			break;
		default:
			return null;
		}
		if(ret!=null) 
			return Arrays.asList(ret);
		else
			return Arrays.asList();
	}
	
	@Override
	public boolean addDeviceOfRole(Port device, int role) {
		if(role == 0 && source == null){
			source = device;
		}else if(role == 1 && destination == source){
			destination = device;
		}else{
			return false;
		}
		return false;
	}
	
	@Override
	public void removeDevice(Port device) {
		if(device == source)
			source = null;
		if(device == destination)
			destination = null;
		
	}

}
